"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var swagger_definition_constant_1 = require("./swagger-definition.constant");
var _ = require("lodash");
var assert = require("assert");
var SwaggerService = /** @class */ (function () {
    function SwaggerService() {
    }
    SwaggerService.getData = function () {
        return _.cloneDeep(SwaggerService.data);
    };
    SwaggerService.setBasePath = function (basePath) {
        SwaggerService.data.basePath = basePath;
    };
    SwaggerService.setOpenapi = function (openapi) {
        SwaggerService.data.openapi = openapi;
    };
    SwaggerService.setInfo = function (info) {
        SwaggerService.data.info = info;
    };
    SwaggerService.setSchemes = function (schemes) {
        SwaggerService.data.schemes = schemes;
    };
    SwaggerService.setProduces = function (produces) {
        SwaggerService.data.produces = produces;
    };
    SwaggerService.setConsumes = function (consumes) {
        SwaggerService.data.consumes = consumes;
    };
    SwaggerService.setHost = function (host) {
        SwaggerService.data.host = host;
    };
    SwaggerService.setDefinitions = function (definitions) {
        SwaggerService.data.definitions = definitions;
    };
    SwaggerService.addPath = function (args, target) {
        var currentController = {
            path: args.path,
            name: args.name,
            paths: {}
        };
        if (args.description) {
            currentController.description = args.description;
        }
        for (var controllerIndex in SwaggerService.controllerMap) {
            var controller = SwaggerService.controllerMap[controllerIndex];
            if (controllerIndex === target.name) {
                currentController = controller;
                currentController.path = args.path;
                currentController.name = args.name;
                currentController.description = args.description;
            }
        }
        SwaggerService.controllerMap[target.name] = currentController;
    };
    SwaggerService.addOperationGet = function (args, target, propertyKey) {
        assert.ok(args, "Args are required.");
        assert.ok(args.responses, "Responses are required.");
        if (args.parameters) {
            assert.ok(!args.parameters.body, "Parameter body is not required.");
        }
        SwaggerService.addOperation("get", args, target, propertyKey);
    };
    SwaggerService.addOperationPost = function (args, target, propertyKey) {
        assert.ok(args, "Args are required.");
        assert.ok(args.parameters, "Parameters are required.");
        assert.ok(args.responses, "Responses are required.");
        SwaggerService.addOperation("post", args, target, propertyKey);
    };
    SwaggerService.addOperationPut = function (args, target, propertyKey) {
        assert.ok(args, "Args are required.");
        assert.ok(args.parameters, "Parameters are required.");
        assert.ok(args.responses, "Responses are required.");
        SwaggerService.addOperation("put", args, target, propertyKey);
    };
    SwaggerService.addOperationPatch = function (args, target, propertyKey) {
        assert.ok(args, "Args are required.");
        assert.ok(args.parameters, "Parameters are required.");
        assert.ok(args.responses, "Responses are required.");
        SwaggerService.addOperation("patch", args, target, propertyKey);
    };
    SwaggerService.addOperationDelete = function (args, target, propertyKey) {
        assert.ok(args, "Args are required.");
        assert.ok(args.parameters, "Parameters are required.");
        assert.ok(!args.parameters.body, "Parameter body is not required.");
        assert.ok(args.responses, "Responses are required.");
        SwaggerService.addOperation("delete", args, target, propertyKey);
    };
    SwaggerService.addOperation = function (operation, args, target, propertyKey) {
        var currentController = {
            paths: {}
        };
        for (var index in SwaggerService.controllerMap) {
            var controller = SwaggerService.controllerMap[index];
            if (index === target.constructor.name) {
                currentController = controller;
            }
        }
        var currentPath;
        if (args.path && args.path.length > 0) {
            if (!currentController.paths[args.path]) {
                currentController.paths[args.path] = {};
            }
            currentPath = currentController.paths[args.path];
            currentPath.path = args.path;
        }
        else {
            if (!currentController.paths["/"]) {
                currentController.paths["/"] = {};
            }
            currentPath = currentController.paths["/"];
        }
        if ("get" === operation) {
            currentPath.get = SwaggerService.buildOperation(args, target, propertyKey);
        }
        if ("post" === operation) {
            currentPath.post = SwaggerService.buildOperation(args, target, propertyKey);
        }
        if ("put" === operation) {
            currentPath.put = SwaggerService.buildOperation(args, target, propertyKey);
        }
        if ("patch" === operation) {
            currentPath.patch = SwaggerService.buildOperation(args, target, propertyKey);
        }
        if ("delete" === operation) {
            currentPath.delete = SwaggerService.buildOperation(args, target, propertyKey);
        }
        SwaggerService.controllerMap[target.constructor.name] = currentController;
    };
    SwaggerService.buildOperation = function (args, target, propertyKey) {
        var operation = {
            operationId: propertyKey,
            tags: []
        };
        if (args.description) {
            operation.description = args.description;
        }
        if (args.summary) {
            operation.summary = args.summary;
        }
        if (args.produces && args.produces.length > 0) {
            operation.produces = args.produces;
        }
        if (args.consumes && args.consumes.length > 0) {
            operation.consumes = args.consumes;
        }
        if (args.parameters) {
            operation.parameters = [];
            if (args.parameters.path) {
                operation.parameters = _.concat(operation.parameters, SwaggerService.buildParameters(swagger_definition_constant_1.SwaggerDefinitionConstant.Parameter.In.PATH, args.parameters.path));
            }
            if (args.parameters.query) {
                operation.parameters = _.concat(operation.parameters, SwaggerService.buildParameters(swagger_definition_constant_1.SwaggerDefinitionConstant.Parameter.In.QUERY, args.parameters.query));
            }
            if (args.parameters.body) {
                assert.ok(args.parameters.body.model, "Definition are required.");
                var newParameterBody = {
                    name: swagger_definition_constant_1.SwaggerDefinitionConstant.Parameter.In.BODY,
                    in: swagger_definition_constant_1.SwaggerDefinitionConstant.Parameter.In.BODY
                };
                if (args.parameters.body.required) {
                    newParameterBody.required = true;
                }
                var swaggerOperationSchema = {
                    $ref: SwaggerService.buildRef(args.parameters.body.model)
                };
                newParameterBody.schema = swaggerOperationSchema;
                operation.parameters.push(newParameterBody);
            }
            if (args.parameters.formData) {
                operation.parameters = _.concat(operation.parameters, SwaggerService.buildParameters(swagger_definition_constant_1.SwaggerDefinitionConstant.Parameter.In.FORM_DATA, args.parameters.formData));
            }
        }
        if (args.responses) {
            operation.responses = {};
            for (var responseIndex in args.responses) {
                var response = args.responses[responseIndex];
                var newSwaggerOperationResponse = {};
                if (response.description) {
                    newSwaggerOperationResponse.description = response.description;
                }
                else {
                    switch (responseIndex) {
                        case "200":
                            newSwaggerOperationResponse.description = "Success";
                            break;
                        case "201":
                            newSwaggerOperationResponse.description = "Success and Created";
                            break;
                        case "204":
                            newSwaggerOperationResponse.description = "Success and 	No Content";
                            break;
                        case "400":
                            newSwaggerOperationResponse.description = "Client error and Bad Request";
                            break;
                        case "401":
                            newSwaggerOperationResponse.description = "Client error and Unauthorized";
                            break;
                        case "404":
                            newSwaggerOperationResponse.description = "Client error and Not Found";
                            break;
                        case "406":
                            newSwaggerOperationResponse.description = "Client error and Not Acceptable";
                            break;
                        default:
                            newSwaggerOperationResponse.description = null;
                    }
                }
                if (response.model) {
                    var ref = SwaggerService.buildRef(response.model);
                    var newSwaggerOperationResponseSchema = {
                        $ref: ref
                    };
                    if (response.isArray) {
                        newSwaggerOperationResponseSchema = {
                            items: {
                                $ref: ref
                            },
                            type: swagger_definition_constant_1.SwaggerDefinitionConstant.Response.Type.ARRAY
                        };
                    }
                    newSwaggerOperationResponse.schema = newSwaggerOperationResponseSchema;
                }
                operation.responses[responseIndex] = newSwaggerOperationResponse;
            }
        }
        return operation;
    };
    SwaggerService.buildParameters = function (type, parameters) {
        var swaggerOperationParameter = [];
        for (var parameterIndex in parameters) {
            var parameter = parameters[parameterIndex];
            var newSwaggerOperationParameter = {
                name: parameterIndex,
                in: type,
                type: parameter.type
            };
            if (parameter.description) {
                newSwaggerOperationParameter.description = parameter.description;
            }
            if (parameter.required) {
                newSwaggerOperationParameter.required = true;
            }
            if (parameter.format) {
                newSwaggerOperationParameter.format = parameter.format;
            }
            if (parameter.deprecated) {
                newSwaggerOperationParameter.deprecated = true;
            }
            if (parameter.allowEmptyValue) {
                newSwaggerOperationParameter.allowEmptyValue = true;
            }
            swaggerOperationParameter.push(newSwaggerOperationParameter);
        }
        return swaggerOperationParameter;
    };
    SwaggerService.buildSwagger = function () {
        var data = _.cloneDeep(SwaggerService.data);
        for (var controllerIndex in SwaggerService.controllerMap) {
            var controller = SwaggerService.controllerMap[controllerIndex];
            for (var pathIndex in controller.paths) {
                var path = controller.paths[pathIndex];
                var swaggerPath = {};
                if (path.get) {
                    swaggerPath.get = path.get;
                    if (!swaggerPath.get.produces) {
                        swaggerPath.get.produces = SwaggerService.data.produces;
                    }
                    if (!swaggerPath.get.consumes) {
                        swaggerPath.get.consumes = SwaggerService.data.consumes;
                    }
                    swaggerPath.get.tags = [_.capitalize(controller.name)];
                }
                if (path.post) {
                    swaggerPath.post = path.post;
                    if (!swaggerPath.post.produces) {
                        swaggerPath.post.produces = SwaggerService.data.produces;
                    }
                    if (!swaggerPath.post.consumes) {
                        swaggerPath.post.consumes = SwaggerService.data.consumes;
                    }
                    swaggerPath.post.tags = [_.capitalize(controller.name)];
                }
                if (path.put) {
                    swaggerPath.put = path.put;
                    if (!swaggerPath.put.produces) {
                        swaggerPath.put.produces = SwaggerService.data.produces;
                    }
                    if (!swaggerPath.put.consumes) {
                        swaggerPath.put.consumes = SwaggerService.data.consumes;
                    }
                    swaggerPath.put.tags = [_.capitalize(controller.name)];
                }
                if (path.patch) {
                    swaggerPath.patch = path.patch;
                    if (!swaggerPath.patch.produces) {
                        swaggerPath.patch.produces = SwaggerService.data.produces;
                    }
                    if (!swaggerPath.patch.consumes) {
                        swaggerPath.patch.consumes = SwaggerService.data.consumes;
                    }
                    swaggerPath.patch.tags = [_.capitalize(controller.name)];
                }
                if (path.delete) {
                    swaggerPath.delete = path.delete;
                    if (!swaggerPath.delete.produces) {
                        swaggerPath.delete.produces = SwaggerService.data.produces;
                    }
                    if (!swaggerPath.delete.consumes) {
                        swaggerPath.delete.consumes = SwaggerService.data.consumes;
                    }
                    swaggerPath.delete.tags = [_.capitalize(controller.name)];
                }
                if (path.path && path.path.length > 0) {
                    data.paths[controller.path.concat(path.path)] = swaggerPath;
                }
                else {
                    data.paths[controller.path] = swaggerPath;
                }
            }
            data.tags.push({
                name: _.capitalize(controller.name), description: controller.description
            });
        }
        SwaggerService.data = data;
    };
    SwaggerService.buildRef = function (definition) {
        return "#/definitions/".concat(_.capitalize(definition));
    };
    SwaggerService.controllerMap = [];
    SwaggerService.definitionsMap = {};
    SwaggerService.data = {
        basePath: "/",
        openapi: "",
        info: {
            title: "",
            contact: {},
            license: {
                name: ""
            },
            version: ""
        },
        paths: {},
        tags: [],
        schemes: [swagger_definition_constant_1.SwaggerDefinitionConstant.Scheme.HTTP],
        produces: [swagger_definition_constant_1.SwaggerDefinitionConstant.Produce.JSON],
        consumes: [swagger_definition_constant_1.SwaggerDefinitionConstant.Consume.JSON],
        definitions: {},
        swagger: "2.0"
    };
    return SwaggerService;
}());
exports.SwaggerService = SwaggerService;
